import { EventEmitter, QueryList, TemplateRef, OnInit } from '@angular/core';
import { Subject } from 'rxjs/Subject';
import { Observable } from 'rxjs/Observable';
import 'rxjs/add/operator/debounceTime';
import 'rxjs/add/operator/do';
import { DataTableColumnDirective } from '../shared/directive/data-table-column.directive';
import { DataTableRowComponent } from './data-table-row/data-table-row.component';
import { IDataTableParams } from '../shared/types/data-table-params.type';
import { RowCallback } from '../shared/types/row-callback.type';
import { IDataTableTranslations } from '../shared/types/data-table-translations.type';
export declare class DataTableComponent implements IDataTableParams, OnInit {
    itemCount: number;
    headerTitle?: string;
    header: boolean;
    pagination: boolean;
    indexColumn: boolean;
    indexColumnHeader: string;
    rowColors?: RowCallback;
    rowTooltip?: RowCallback;
    selectColumn: boolean;
    multiSelect: boolean;
    substituteRows: boolean;
    expandableRows: boolean;
    translations: IDataTableTranslations;
    selectOnRowClick: boolean;
    autoReload: boolean;
    showReloading: boolean;
    noDataMessage?: string;
    additionalClass: string;
    items: any[];
    params: IDataTableParams;
    sortBy: string | undefined;
    sortAsc: boolean | undefined;
    offset: number;
    limit: number;
    page: number;
    readonly lastPage: number;
    reload: EventEmitter<{}>;
    rowClick: EventEmitter<{}>;
    rowDoubleClick: EventEmitter<{}>;
    headerClick: EventEmitter<{}>;
    cellClick: EventEmitter<{}>;
    rows?: QueryList<DataTableRowComponent>;
    expandTemplate?: TemplateRef<any>;
    columns?: QueryList<DataTableColumnDirective>;
    indexColumnVisible?: boolean;
    selectColumnVisible?: boolean;
    expandColumnVisible?: boolean;
    selectedRow?: DataTableRowComponent;
    selectedRows: DataTableRowComponent[];
    resizeLimit: number;
    _displayParams: IDataTableParams;
    readonly displayParams: IDataTableParams;
    _reloading: boolean;
    readonly reloading: boolean;
    subject$: Subject<void>;
    stream$: Observable<void>;
    private _resizeInProgress;
    private _items;
    private _sortBy?;
    private _sortAsc?;
    private _offset;
    private _limit;
    private _selectAllCheckbox;
    selectAllCheckbox: boolean;
    constructor();
    sort(sortBy: string, asc: boolean): void;
    ngOnInit(): void;
    reloadItems(): void;
    _updateDisplayParams(): void;
    rowClicked(row: DataTableRowComponent, event: Event): void;
    rowDoubleClicked(row: DataTableRowComponent, event: Event): void;
    headerClicked(column: DataTableColumnDirective, event: Event): void;
    getRowColor(item: any, index: number, row: DataTableRowComponent): string | undefined;
    onRowSelectChanged(row: DataTableRowComponent): void;
    private _initDefaultValues();
    private _initDefaultClickEvents();
    private _onReloadFinished();
    private cellClicked(column, row, event);
    private _getRemoteParameters();
    private sortColumn(column);
    readonly columnCount: number;
    private _onSelectAllChanged(value);
    readonly substituteItems: {}[];
    private resizeColumnStart(event, column, columnElement);
    private _isResizeInLimit(columnElement, dx);
}
